<?php
/**
 * Loulou66
 * LpsNewProducts module for Prestashop
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php*
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    Loulou66.fr <contact@loulou66.fr>
 *  @copyright loulou66.fr
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */
use PrestaShop\PrestaShop\Adapter\Image\ImageRetriever;
use PrestaShop\PrestaShop\Adapter\Product\PriceFormatter;
use PrestaShop\PrestaShop\Adapter\Product\ProductColorsRetriever;
use PrestaShop\PrestaShop\Core\Module\WidgetInterface;
use PrestaShop\PrestaShop\Core\Product\ProductListingPresenter;

if (!defined('_PS_VERSION_')) {
    exit;
}

class LpsNewProducts extends Module implements WidgetInterface
{
    public $templateFile;

    public function __construct()
    {
        $this->name = 'lpsnewproducts';
        $this->author = 'Loulou66';
        $this->version = '8.0.1';
        $this->tab = 'front_office_features';
        $this->need_instance = 0;
        $this->ps_versions_compliancy = ['min' => '1.7.0', 'max' => _PS_VERSION_];
        $this->bootstrap = true;
        parent::__construct();
        $this->displayName = $this->l('LPS New Products Carousel');
        $this->description = $this->l('Add a block displaying your store newest products as a CAROUSEL');
        $this->templateFile = 'module:lpsnewproducts/views/templates/hook/lpsnewproducts.tpl';
        $this->module_key = 'ec1549d2eea25fd0d6a3bd647eb07106';
    }
    public function install()
    {
        $this->_clearCache('*');
        return parent::install()
            && Configuration::updateValue('LPSNEWPRODUCTS_ENABLE', 1)
            && Configuration::updateValue('LPSNEWPRODUCTS_PROD_DISPLAY', 20)
            && Configuration::updateValue('LPSNEWPRODUCTS_PROD_PER_ROW', 4)
            && Configuration::updateValue('LPSNEWPRODUCTS_LOOP', 1)
            && Configuration::updateValue('LPSNEWPRODUCTS_AUTOPLAY', 1)
            && Configuration::updateValue('LPSNEWPRODUCTS_RANDOM', 0)
            && $this->registerHook('actionAdminControllerSetMedia')
            && $this->registerHook('actionFrontControllerSetMedia')
            && $this->registerHook('actionProductAdd')
            && $this->registerHook('actionProductUpdate')
            && $this->registerHook('actionProductDelete')
            && $this->registerHook('displayHome')
            && $this->registerHook('displayFooterProduct');
    }
    public function uninstall()
    {
        $this->_clearCache('*');
        return parent::uninstall()
            && Configuration::deleteByName('LPSNEWPRODUCTS_ENABLE')
            && Configuration::deleteByName('LPSNEWPRODUCTS_PROD_DISPLAY')
            && Configuration::deleteByName('LPSNEWPRODUCTS_PROD_PER_ROW')
            && Configuration::deleteByName('LPSNEWPRODUCTS_LOOP')
            && Configuration::deleteByName('LPSNEWPRODUCTS_AUTOPLAY')
            && Configuration::deleteByName('LPSNEWPRODUCTS_RANDOM');
    }
    public function isUsingNewTranslationSystem()
    {
        return false;
    }
    public function hookActionProductAdd($params)
    {
        $this->_clearCache('*');
    }
    public function hookActionProductUpdate($params)
    {
        $this->_clearCache('*');
    }
    public function hookActionProductDelete($params)
    {
        $this->_clearCache('*');
    }
    public function _clearCache($template, $cache_id = null, $compile_id = null)
    {
        return parent::_clearCache($this->templateFile);
    }
    public function hookActionAdminControllerSetMedia($params)
    {
        if ($this->context->controller->controller_name == 'AdminModules'
            && Tools::getValue('configure') == $this->name
        ) {
            $this->context->controller->addCSS($this->_path . 'views/css/' . $this->name . 'admin.css');
            $this->context->controller->addJS($this->_path . 'views/js/' . $this->name . 'admin.js');
        }
    }
    public function getContent()
    {
        $html = '';
        if (Tools::isSubmit('submitlpsnewproducts')) {
            if (!Validate::isInt((int) Tools::getValue('LPSNEWPRODUCTS_PROD_DISPLAY'))
                || (int) Tools::getValue('LPSNEWPRODUCTS_PROD_DISPLAY') <= 0
            ) {
                $html .= $this->displayError(
                    $this->l('The « Products to display » field is invalid. Please enter a positive number.')
                );
            } elseif (!Validate::isInt((int) Tools::getValue('PS_NB_DAYS_NEW_PRODUCT'))
                || (int) Tools::getValue('PS_NB_DAYS_NEW_PRODUCT') <= 0
            ) {
                $html .= $this->displayError(
                    $this->l('The « Number of days » field is invalid. Please enter a positive number.')
                );
            } else {
                Configuration::updateValue('LPSNEWPRODUCTS_ENABLE', (int) Tools::getValue('LPSNEWPRODUCTS_ENABLE'));
                Configuration::updateValue('PS_NB_DAYS_NEW_PRODUCT', (int) Tools::getValue('PS_NB_DAYS_NEW_PRODUCT'));
                Configuration::updateValue(
                    'LPSNEWPRODUCTS_PROD_DISPLAY',
                    (int) Tools::getValue('LPSNEWPRODUCTS_PROD_DISPLAY')
                );
                Configuration::updateValue(
                    'LPSNEWPRODUCTS_PROD_PER_ROW',
                    (int) Tools::getValue('LPSNEWPRODUCTS_PROD_PER_ROW')
                );
                Configuration::updateValue('LPSNEWPRODUCTS_LOOP', (int) Tools::getValue('LPSNEWPRODUCTS_LOOP'));
                Configuration::updateValue('LPSNEWPRODUCTS_AUTOPLAY', (int) Tools::getValue('LPSNEWPRODUCTS_AUTOPLAY'));
                Configuration::updateValue('LPSNEWPRODUCTS_RANDOM', (int) Tools::getValue('LPSNEWPRODUCTS_RANDOM'));
                $this->_clearCache('*');
                $html .= $this->displayConfirmation($this->l('Settings updated'));
            }
        }
        return $html . $this->renderForm();
    }
    public function renderForm()
    {
        $fields_form = [
            'form' => [
                'legend' => [
                    'title' => $this->l('Settings'),
                    'icon' => 'icon-cogs',
                ],
                'input' => [
                    [
                        'type' => 'switch',
                        'label' => $this->l('Enable carousel'),
                        'name' => 'LPSNEWPRODUCTS_ENABLE',
                        'is_bool' => true,
                        'values' => [
                           [
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Number of days for which the product is considered "new"'),
                        'name' => 'PS_NB_DAYS_NEW_PRODUCT',
                        'desc' => $this->l('Determine the number of days for which the product is considered "new"'),
                        'hint' => $this->l('Determine the number of days for which the product is considered "new"'),
                        'form_group_class' => 'LPSNEWPRODUCTS_ENABLE',
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Products to display'),
                        'name' => 'LPSNEWPRODUCTS_PROD_DISPLAY',
                        'desc' => $this->l('Determine the total number of products to display in the carousel'),
                        'hint' => $this->l('Determine the total number of products to display in the carousel'),
                        'form_group_class' => 'LPSNEWPRODUCTS_ENABLE',
                    ],
                    [
                        'type' => 'html',
                        'label' => $this->l('Products visible'),
                        'html_content' => $this->displayProdPerRow(),
                        'name' => '',
                        'desc' => $this->l('Define the number of products visible'),
                        'hint' => $this->l('Define the number of products visible'),
                        'form_group_class' => 'LPSNEWPRODUCTS_ENABLE',
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('Loop'),
                        'name' => 'LPSNEWPRODUCTS_LOOP',
                        'is_bool' => true,
                        'form_group_class' => 'LPSNEWPRODUCTS_ENABLE',
                        'values' => [
                           [
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('Autoplay'),
                        'name' => 'LPSNEWPRODUCTS_AUTOPLAY',
                        'is_bool' => true,
                        'form_group_class' => 'LPSNEWPRODUCTS_ENABLE',
                        'values' => [
                           [
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('Random display'),
                        'name' => 'LPSNEWPRODUCTS_RANDOM',
                        'hint' => $this->l('Enable if you wish the products to be displayed randomly'),
                        'desc' => $this->l('Enable if you wish the products to be displayed randomly'),
                        'is_bool' => true,
                        'form_group_class' => 'LPSNEWPRODUCTS_ENABLE',
                        'values' => [
                           [
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                    ],
                ],
                'submit' => [
                    'title' => $this->l('Save'),
                ],
            ],
        ];
        $lang = new Language((int) Configuration::get('PS_LANG_DEFAULT'));
        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG')
            ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG')
            : 0;
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitlpsnewproducts';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false) .
            '&configure=' . $this->name .
            '&tab_module=' . $this->tab .
            '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = [
            'fields_value' => $this->getConfigFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        ];
        return $helper->generateForm([$fields_form]);
    }
    public function getConfigFieldsValues()
    {
        return [
            'LPSNEWPRODUCTS_ENABLE' => (int) Tools::getValue(
                'LPSNEWPRODUCTS_ENABLE',
                Configuration::get('LPSNEWPRODUCTS_ENABLE'),
            ),
            'PS_NB_DAYS_NEW_PRODUCT' => (int) Tools::getValue(
                'PS_NB_DAYS_NEW_PRODUCT',
                Configuration::get('PS_NB_DAYS_NEW_PRODUCT'),
            ),
            'LPSNEWPRODUCTS_PROD_DISPLAY' => (int) Tools::getValue(
                'LPSNEWPRODUCTS_PROD_DISPLAY',
                Configuration::get('LPSNEWPRODUCTS_PROD_DISPLAY'),
            ),
            'LPSNEWPRODUCTS_LOOP' => (int) Tools::getValue(
                'LPSNEWPRODUCTS_LOOP',
                Configuration::get('LPSNEWPRODUCTS_LOOP'),
            ),
            'LPSNEWPRODUCTS_AUTOPLAY' => (int) Tools::getValue(
                'LPSNEWPRODUCTS_AUTOPLAY',
                Configuration::get('LPSNEWPRODUCTS_AUTOPLAY'),
            ),
            'LPSNEWPRODUCTS_RANDOM' => (int) Tools::getValue(
                'LPSNEWPRODUCTS_RANDOM',
                Configuration::get('LPSNEWPRODUCTS_RANDOM'),
            ),
        ];
    }
    public function displayProdPerRow()
    {
        $tpl = _PS_MODULE_DIR_ . $this->name . '/views/templates/admin/lpsnpprodperrow.tpl';
        $lpsnpprodperrow = $this->context->smarty->createTemplate($tpl);
        $lpsnpprodperrow->assign(
            [
                'lpsnp_prodperrow' => (int) Tools::getValue(
                    'LPSNEWPRODUCTS_PROD_PER_ROW',
                    Configuration::get('LPSNEWPRODUCTS_PROD_PER_ROW')
                ),
            ],
        );
        return $lpsnpprodperrow->fetch();
    }
    public function hookActionFrontControllerSetMedia()
    {
        if (!$this->active || !Configuration::get('LPSNEWPRODUCTS_ENABLE')) {
            return false;
        }
        Media::addJsDef(
            [
                'lpsnp_productsPerRow' => Configuration::get('LPSNEWPRODUCTS_PROD_PER_ROW'),
                'lpsnp_loop' => Configuration::get('LPSNEWPRODUCTS_LOOP') ? true : false,
                'lpsnp_autoplay' => Configuration::get('LPSNEWPRODUCTS_AUTOPLAY') ? true : false,
            ]
        );
        $this->context->controller->registerStylesheet(
            'modules-lpsnewproductsfrontcss',
            'modules/' . $this->name . '/views/css/lpsnewproductsfront.css'
        );
        $this->context->controller->registerStylesheet(
            'modules-owlcarouselcss',
            'modules/' . $this->name . '/libs/owlcarousel/css/owl.carousel.css'
        );
        $this->context->controller->registerStylesheet(
            'modules-owlcarouselthemecss',
            'modules/' . $this->name . '/libs/owlcarousel/css/owl.theme.default.css'
        );
        $this->context->controller->registerJavascript(
            'modules-owlcarouseljs',
            'modules/' . $this->name . '/libs/owlcarousel/js/owl.carousel.min.js'
        );
        $this->context->controller->registerJavascript(
            'modules-lpsnewproductsfrontjs',
            'modules/' . $this->name . '/views/js/lpsnewproducts.js'
        );
    }
    public function renderWidget($hookName, array $configuration)
    {
        if (!$this->active || !Configuration::get('LPSNEWPRODUCTS_ENABLE')) {
            return false;
        }
        $lpsnewproductsrandom = Configuration::get('LPSNEWPRODUCTS_RANDOM');
        if (!$lpsnewproductsrandom) {
            if (!$this->isCached($this->templateFile, $this->getCacheId('lpsnewproducts'))) {
                $variables = $this->getWidgetVariables($hookName, $configuration);
                if (empty($variables)) {
                    return false;
                }
                $this->context->smarty->assign($variables);
            }
            return $this->fetch($this->templateFile, $this->getCacheId('lpsnewproducts'));
        } else {
            $variables = $this->getWidgetVariables($hookName, $configuration);
            if (empty($variables)) {
                return false;
            }
            $this->context->smarty->assign($variables);
            return $this->fetch($this->templateFile);
        }
    }
    public function getWidgetVariables($hookName, array $configuration)
    {
        $lpsnewproducts = $this->getNewProducts();
        if (count($lpsnewproducts) <= Configuration::get('LPSNEWPRODUCTS_PROD_PER_ROW')) {
            $this->context->smarty->assign(['lpsnp_hidenav' => true]);
        }
        if (!empty($lpsnewproducts)) {
            return ['lpsnewproducts' => $lpsnewproducts];
        }
        return false;
    }
    protected function getNewProducts()
    {
        $lpsnewproducts = false;
        $nbProducts = Configuration::get('LPSNEWPRODUCTS_PROD_DISPLAY');
        if (Configuration::get('PS_NB_DAYS_NEW_PRODUCT')) {
            $lpsnewproducts = $this->getProductsNew(
                (int) $this->context->shop->id,
                (int) $this->context->language->id,
                $nbProducts
            );
        }
        $assembler = new ProductAssembler($this->context);
        $presenterFactory = new ProductPresenterFactory($this->context);
        $presentationSettings = $presenterFactory->getPresentationSettings();
        $presenter = new ProductListingPresenter(
            new ImageRetriever($this->context->link),
            $this->context->link,
            new PriceFormatter(),
            new ProductColorsRetriever(),
            $this->context->getTranslator()
        );
        $products_for_template = [];
        if (is_array($lpsnewproducts)) {
            if (Configuration::get('LPSNEWPRODUCTS_RANDOM')) {
                shuffle($lpsnewproducts);
            }
            foreach ($lpsnewproducts as $rawProduct) {
                $products_for_template[] = $presenter->present(
                    $presentationSettings,
                    $assembler->assembleProduct($rawProduct),
                    $this->context->language
                );
            }
        }
        return $products_for_template;
    }
    public function getProductsNew($id_shop, $id_lang, $nb_products = 10)
    {
        $context = Context::getContext();
        $front = true;
        $page_number = 1;
        $order_by = 'date_add';
        $order_way = 'DESC';
        $order_by_prefix = 'product_shop';
        $nb_days_new_product = (int) Configuration::get('PS_NB_DAYS_NEW_PRODUCT')
            ? (int) Configuration::get('PS_NB_DAYS_NEW_PRODUCT')
            : 20;
        $sql = new DbQuery();
        $sql->select(
            'p.*, product_shop.*, stock.out_of_stock, IFNULL(stock.quantity, 0) as quantity, pl.`description`,
            pl.`description_short`, pl.`link_rewrite`, pl.`meta_description`, pl.`meta_keywords`, pl.`meta_title`,
            pl.`name`, pl.`available_now`, pl.`available_later`, image_shop.`id_image` id_image, il.`legend`,
            m.`name` AS manufacturer_name,
            product_shop . `date_add` > "' . date('Y-m-d', strtotime('-' . $nb_days_new_product . ' DAY')) . '" as new'
        );
        $sql->from('product', 'p');
        $sql->join(Shop::addSqlAssociation('product', 'p'));
        $sql->leftJoin(
            'product_lang',
            'pl',
            'p.`id_product` = pl . `id_product`
                AND pl . `id_lang` = ' . (int) $id_lang . Shop::addSqlRestrictionOnLang('pl')
        );
        $sql->leftJoin(
            'image_shop',
            'image_shop',
            'image_shop.`id_product` = p.`id_product`
                AND image_shop.cover=1 AND image_shop.id_shop=' . (int) $id_shop
        );
        $sql->leftJoin(
            'image_lang',
            'il',
            'image_shop.`id_image` = il.`id_image` AND il.`id_lang` = ' . (int) $id_lang
        );
        $sql->leftJoin('manufacturer', 'm', 'm.`id_manufacturer` = p.`id_manufacturer`');
        $sql->where('product_shop.`active` = 1');
        if ($front) {
            $sql->where('product_shop.`visibility` IN ("both", "catalog")');
        }
        $sql->where(
            'product_shop.`date_add` > "' . date('Y-m-d', strtotime('-' . $nb_days_new_product . ' DAY')) . '"'
        );
        if (Group::isFeatureActive()) {
            $groups = FrontController::getCurrentCustomerGroups();
            $sqlgroup = count($groups) ? 'IN (' . implode(',', $groups) . ')' : '=' . (int) Group::getCurrent()->id;
            $sqlWhere = 'EXISTS(
                SELECT 1
                FROM `' . _DB_PREFIX_ . 'category_product` cp
                JOIN `' . _DB_PREFIX_ . 'category_group` cg ON (cp.`id_category` = cg.`id_category`)
                WHERE cp.`id_product` = p.`id_product`
                AND cg.`id_group` ' . $sqlgroup .
            ')';
            $sql->where($sqlWhere);
        }
        $sql->orderBy(pSQL($order_by_prefix) . '.`' . pSQL($order_by) . '` ' . pSQL($order_way));
        $sql->limit($nb_products, (int) (($page_number - 1) * $nb_products));
        if (Combination::isFeatureActive()) {
            $sql->select('product_attribute_shop.minimal_quantity AS product_attribute_minimal_quantity, IFNULL(product_attribute_shop.id_product_attribute,0) id_product_attribute');
            $sql->leftJoin('product_attribute_shop', 'product_attribute_shop', 'p.`id_product` = product_attribute_shop.`id_product` AND product_attribute_shop.`default_on` = 1 AND product_attribute_shop.id_shop=' . (int) $id_shop);
        }
        $sql->join(Product::sqlStock('p', 0));
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);
        if (!$result) {
            return false;
        }
        return Product::getProductsProperties((int) $id_lang, $result);
    }
}
